package com.fluxloop.pinch

import com.fluxloop.pinch.sdk.Pinch
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

class PinchPlugin: MethodCallHandler {
  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "com.fluxloop.pinch/sdk")
      channel.setMethodCallHandler(PinchPlugin())
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    try {
      when (call.method) {
        "start" -> {
          Pinch.start()
          result.success(true)
        }
        "stop" -> {
          Pinch.stop()
          result.success(true)
        }
        "setApiKey" -> {
          val apiKey: String? = call.argument("apiKey")
          if (apiKey != null) {
            Pinch.apiKey = apiKey
            result.success(true)
          } else {
            result.error("Invalid API key supplied", call.method, null)
          }
        }
        "setProductionMode" -> {
          val productionMode: Boolean? = call.argument("enabled")
          Pinch.productionMode = productionMode != false
          result.success(true)
        }
        "setMessagingId" -> {
          val messagingId: String? = call.argument("messagingId")
          Pinch.messagingId = messagingId
          result.success(true)
        }
        "sendCustomEvent" -> {
          val type: String = call.argument("type")
          val json: String? = call.argument("json")
          Pinch.sendCustomEvent(type, json)
          result.success(true)
        }
        "sendDemographicProfile" -> {
          val birthYear: Int? = call.argument("birthYear")
          val gender: String? = call.argument("gender")
          if (birthYear == null || gender == null) {
            result.error("Received invalid data", call.method, null);
          } else {
            val gender = when(gender) {
              "male" -> DemographicProfile.Gender.MALE
              "female" -> DemographicProfile.Gender.FEMALE
              "unknown" -> DemographicProfile.Gender.UNKNOWN
            }
            Pinch.sendDemographicProfile(birthYear, gender)
            result.success(true)
          }
        }
      }
    } catch (e: Exception) {
      result.error("Exception encountered", call.method, e)
    }
  }
}
