import Flutter
import UIKit
import PureSDK

public class SwiftPinchPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "com.fluxloop.pinch/sdk", binaryMessenger: registrar.messenger())
    let instance = SwiftPinchPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "start":
        Pure.startTracking()
        break
    case "stop":
        Pure.stopTracking()
        break
    case "setApiKey":
        break
    case "setProductionMode":
        break
    case "setMessagingId":
        break
    case "sendCustomEvent":
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError.init(code: "INVALID_ARGS", message: "Invalid arguments supplied", details: nil))
            return
        }
        let json = args["json"] as? String
        if (json == nil) {
            result(false)
            return
        }
        if let dict = (convertToDictionary(json: json!)) {
         Pure.associateMetadata(withType: "DemographicProfile", payload: dict, force: true, success: {
                result(true)
                return
            }) { (error) in
                result(false)
                return
            }
        }
        break
    case "sendDemographicProfile":
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError.init(code: "INVALID_ARGS", message: "Invalid arguments supplied", details: nil))
            return
        }
        let birthYear = args["birthYear"] as? Int
        let gender = args["gender"] as? String
        
        if (birthYear == nil || gender == nil) {
            result(false)
            return
        }
        
        var dict = Dictionary<String, Any>()
        dict["birthYear"] = birthYear
        dict["gender"] = gender
    
        Pure.createEvent(withType: "DemographicProfile", payload: dict, force: true, success: {
            result(true)
            return
        }) { (error) in
            result(false)
            return
        }
        break
    default:
        result(FlutterError.init(code: "INVALID_METHOD", message: "Method does not exist", details: nil))
    }
    result(true)
  }
    
    private func convertToDictionary(json: String) -> [String: Any]? {
        if let data = json.data(using: .utf8) {
            do {
                return try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any]
            } catch {
              //
            }
        }
        return nil
    }
    
    private func genderToInt(gender: String) -> Int {
        switch gender {
        case "male":
            return 1
        case "female":
            return 2
        default:
            return 0
        }
    }
}
